@@ -31,7 +31,20 @@ removes = []
 WANT_STACK_TRACE = getattr(settings, 'DEBUG_TOOLBAR_MONGO_STACKTRACES', True)
 def _get_stacktrace():
     if WANT_STACK_TRACE:
-        return _tidy_stacktrace(reversed(inspect.stack()))
+        try:
+            stack = inspect.stack()
+        except IndexError:
+            # this is a work around because python's inspect.stack() sometimes fail
+            # when jinja templates are on the stack
+            return [(
+                "",
+                0,
+                "Error retrieving stack",
+                "Could not retrieve stack. IndexError exception occured in inspect.stack(). "
+                "This error might occur when jinja2 templates is on the stack.",
+            )]
+
+        return _tidy_stacktrace(reversed(stack))
     else:
         return []
 
@@ -44,7 +57,9 @@ def _insert(collection_self, doc_or_docs, manipulate=True,
     result = _original_methods['insert'](
         collection_self,
         doc_or_docs,
+        manipulate=manipulate,
         safe=safe,
+        check_keys=check_keys,
         **kwargs
     )
     total_time = (time.time() - start_time) * 1000
@@ -153,7 +168,7 @@ def _cursor_refresh(cursor_self):
         # Normal Query
         query_data['skip'] = privar('skip')
         query_data['limit'] = privar('limit')
-        query_data['query'] = query_son['$query']
+        query_data['query'] = query_son.get('$query')
         query_data['ordering'] = _get_ordering(query_son)
 
     queries.append(query_data)
